#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_pim
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages configuration of a PIM instance.
description:
- Manages configuration of a Protocol Independent Multicast (PIM) instance.
notes:
- Unsupported for Cisco MDS
version_added: 1.0.0
author: Gabriele Gerbino (@GGabriele)
options:
  bfd:
    description:
    - Enables BFD on all PIM interfaces.
    - "Dependency: ''feature bfd''"
    type: str
    choices:
    - enable
    - disable
  ssm_range:
    description:
    - Configure group ranges for Source Specific Multicast (SSM). Valid values are
      multicast addresses or the keyword C(none) or keyword C(default). C(none) removes
      all SSM group ranges. C(default) will set ssm_range to the default multicast
      address. If you set multicast address, please ensure that it is not the same
      as the C(default), otherwise use the C(default) option.
    type: list
    default: []
    elements: str
"""
EXAMPLES = """
- name: Configure ssm_range, enable bfd
  cisco.nxos.nxos_pim:
    bfd: enable
    ssm_range: 224.0.0.0/8

- name: Set to default
  cisco.nxos.nxos_pim:
    ssm_range: default

- name: Remove all ssm group ranges
  cisco.nxos.nxos_pim:
    ssm_range: none
"""

RETURN = """
commands:
    description: commands sent to the device
    returned: always
    type: list
    sample:
      - ip pim bfd
      - ip pim ssm range 224.0.0.0/8
"""


import re

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.config import (
    CustomNetworkConfig,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_config,
    load_config,
)


PARAM_TO_COMMAND_KEYMAP = {
    "bfd": "ip pim bfd",
    "ssm_range": "ip pim ssm range",
}


def get_existing(module, args):
    existing = {}
    config = str(get_config(module))

    for arg in args:
        if "ssm_range" in arg:
            # <value> may be 'n.n.n.n/s', 'none', or 'default'
            m = re.search(
                r"ssm range (?P<value>(?:[\s\d.\/]+|none|default))?$",
                config,
                re.M,
            )
            if m:
                # Remove rsvd SSM range
                value = m.group("value").replace("232.0.0.0/8", "")
                existing[arg] = value.split()

        elif "bfd" in arg and "ip pim bfd" in config:
            existing[arg] = "enable"

    return existing


def apply_key_map(key_map, table):
    new_dict = {}
    for key, value in table.items():
        new_key = key_map.get(key)
        if value is not None:
            new_dict[new_key] = value
    return new_dict


def get_commands(module, existing, proposed, candidate):
    commands = list()
    proposed_commands = apply_key_map(PARAM_TO_COMMAND_KEYMAP, proposed)

    for key, value in proposed_commands.items():
        command = ""
        if key == "ip pim ssm range":
            if value == "default":
                # no cmd needs a value but the actual value does not matter
                command = "no ip pim ssm range none"
            elif value == "none":
                command = "ip pim ssm range none"
            elif value:
                command = "ip pim ssm range {0}".format(value)
        elif key == "ip pim bfd":
            no_cmd = "no " if value == "disable" else ""
            command = no_cmd + key

        if command:
            commands.append(command)

    if commands:
        candidate.add(commands, parents=[])


def main():
    argument_spec = dict(
        bfd=dict(required=False, type="str", choices=["enable", "disable"]),
        ssm_range=dict(required=False, type="list", default=[], elements="str"),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    warnings = list()
    result = {"changed": False, "commands": [], "warnings": warnings}

    params = module.params
    args = [k for k in PARAM_TO_COMMAND_KEYMAP.keys() if params[k] is not None]

    # SSM syntax check
    if "ssm_range" in args:
        for item in params["ssm_range"]:
            if re.search("none|default", item):
                break
            if len(item.split(".")) != 4:
                module.fail_json(
                    msg="Valid ssm_range values are multicast addresses "
                    "or the keyword 'none' or the keyword 'default'.",
                )

    existing = get_existing(module, args)
    proposed_args = dict((k, v) for k, v in params.items() if k in args)

    proposed = {}
    for key, value in proposed_args.items():
        if key == "ssm_range":
            if value and value[0] == "default":
                if existing.get(key):
                    proposed[key] = "default"
            else:
                v = sorted(set([str(i) for i in value]))
                ex = sorted(set([str(i) for i in existing.get(key, [])]))
                if v != ex:
                    proposed[key] = " ".join(str(s) for s in v)

        elif key == "bfd":
            if value != existing.get("bfd", "disable"):
                proposed[key] = value

        elif value != existing.get(key):
            proposed[key] = value

    candidate = CustomNetworkConfig(indent=3)
    get_commands(module, existing, proposed, candidate)

    if candidate:
        candidate = candidate.items_text()
        result["commands"] = candidate
        result["changed"] = True
        load_config(module, candidate)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#
# Copyright (c) 2020, SCC France, Eric Belhomme <ebelhomme@fr.scc.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author:
  - Eric Belhomme (@eric-belhomme) <ebelhomme@fr.scc.com>
version_added: '0.2.0'
name: etcd3
short_description: Get key values from etcd3 server
description:
  - Retrieves key values and/or key prefixes from etcd3 server using its native gRPC API.
  - Try to reuse M(community.general.etcd3) options for connection parameters, but add support for some E(ETCDCTL_*) environment
    variables.
  - See U(https://github.com/etcd-io/etcd/tree/master/Documentation/op-guide) for etcd overview.
options:
  _terms:
    description:
      - The list of keys (or key prefixes) to look up on the etcd3 server.
    type: list
    elements: str
    required: true
  prefix:
    description:
      - Look for key or prefix key.
    type: bool
    default: false
  endpoints:
    description:
      - Counterpart of E(ETCDCTL_ENDPOINTS) environment variable. Specify the etcd3 connection with an URL form, for example
        V(https://hostname:2379), or V(<host>:<port>) form.
      - The V(host) part is overwritten by O(host) option, if defined.
      - The V(port) part is overwritten by O(port) option, if defined.
      - Note that specifying V(https://) in the endpoint URL does not by itself enable TLS. To connect to an HTTPS etcd3
        endpoint, you must provide O(ca_cert) (and optionally O(cert_cert) and O(cert_key)).
    env:
      - name: ETCDCTL_ENDPOINTS
    default: '127.0.0.1:2379'
    type: str
  host:
    description:
      - Etcd3 listening client host.
      - Takes precedence over O(endpoints).
      - Must be a bare hostname or IP address without a URL scheme. If a V(https://) or V(http://) prefix is present,
        it will be stripped automatically.
    type: str
  port:
    description:
      - Etcd3 listening client port.
      - Takes precedence over O(endpoints).
    type: int
  ca_cert:
    description:
      - Etcd3 CA authority.
      - Required to enable TLS when connecting to an HTTPS etcd3 endpoint.
    env:
      - name: ETCDCTL_CACERT
    type: str
  cert_cert:
    description:
      - Etcd3 client certificate.
    env:
      - name: ETCDCTL_CERT
    type: str
  cert_key:
    description:
      - Etcd3 client private key.
    env:
      - name: ETCDCTL_KEY
    type: str
  timeout:
    description:
      - Client timeout.
    default: 60
    env:
      - name: ETCDCTL_DIAL_TIMEOUT
    type: int
  user:
    description:
      - Authenticated user name.
    env:
      - name: ETCDCTL_USER
    type: str
  password:
    description:
      - Authenticated user password.
    env:
      - name: ETCDCTL_PASSWORD
    type: str

notes:
  - O(host) and O(port) options take precedence over O(endpoints) option.
  - The recommended way to connect to etcd3 server is using E(ETCDCTL_ENDPOINT) environment variable and keep O(endpoints),
    O(host), and O(port) unused.
  - To connect to an HTTPS etcd3 endpoint, the O(ca_cert) option must be provided. Merely specifying V(https://) in
    O(endpoints) is not sufficient to enable TLS.
seealso:
  - module: community.general.etcd3
  - plugin: community.general.etcd
    plugin_type: lookup

requirements:
  - "etcd3 >= 0.10"
"""

EXAMPLES = r"""
- name: "a value from a locally running etcd"
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.etcd3', 'foo/bar') }}"

- name: "values from multiple folders on a locally running etcd"
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.etcd3', 'foo', 'bar', 'baz') }}"

- name: "look for a key prefix"
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.etcd3', '/foo/bar', prefix=True) }}"

- name: "connect to etcd3 with a client certificate"
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.etcd3', 'foo/bar', cert_cert='/etc/ssl/etcd/client.pem', cert_key='/etc/ssl/etcd/client.key') }}"

- name: "connect to etcd3 over HTTPS"
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.etcd3', 'foo/bar', endpoints='https://etcd.example.com:2379', ca_cert='/etc/ssl/etcd/ca.pem') }}"
"""

RETURN = r"""
_raw:
  description:
    - List of keys and associated values.
  type: list
  elements: dict
  contains:
    key:
      description: The element's key.
      type: str
    value:
      description: The element's value.
      type: str
"""

import re

from ansible.errors import AnsibleLookupError
from ansible.module_utils.basic import missing_required_lib
from ansible.module_utils.common.text.converters import to_native
from ansible.plugins.lookup import LookupBase
from ansible.utils.display import Display

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms

try:
    import etcd3

    HAS_ETCD = True
except ImportError:
    HAS_ETCD = False

display = Display()

etcd3_cnx_opts = (
    "host",
    "port",
    "ca_cert",
    "cert_key",
    "cert_cert",
    "timeout",
    "user",
    "password",
    # 'grpc_options' Etcd3Client() option currently not supported by lookup module (maybe in future ?)
)


def etcd3_client(client_params):
    try:
        etcd = etcd3.client(**client_params)
        etcd.status()
    except Exception as exp:
        raise AnsibleLookupError(f"Cannot connect to etcd cluster: {exp}") from exp
    return etcd


class LookupModule(LookupBase):
    def run(self, terms, variables, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)

        if not HAS_ETCD:
            display.error(missing_required_lib("etcd3"))
            return None

        # create the etcd3 connection parameters dict to pass to etcd3 class
        client_params = {}

        # etcd3 class expects host and port as connection parameters, so endpoints
        # must be mangled a bit to fit in this scheme.
        # so here we use a regex to extract server and port
        is_https = False
        match = re.compile(
            r"^(?P<scheme>https?://)?(?P<host>(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})|([-_\d\w\.]+))(:(?P<port>\d{1,5}))?/?$"
        ).match(self.get_option("endpoints"))
        if match:
            if match.group("host"):
                client_params["host"] = match.group("host")
            if match.group("port"):
                client_params["port"] = match.group("port")
            if match.group("scheme") and match.group("scheme").startswith("https"):
                is_https = True

        for opt in etcd3_cnx_opts:
            if self.get_option(opt):
                client_params[opt] = self.get_option(opt)

        # strip URL scheme from host if present
        if "host" in client_params:
            host_match = re.match(r"^(?P<scheme>https?://)(?P<host>.+)$", client_params["host"])
            if host_match:
                display.warning(
                    f"The host option contained a URL scheme '{host_match.group('scheme')}' which has been removed. "
                    "Use a bare hostname or IP address for the host option. "
                    "To enable TLS, provide the ca_cert option."
                )
                client_params["host"] = host_match.group("host")
                if host_match.group("scheme").startswith("https"):
                    is_https = True

        if is_https and not self.get_option("ca_cert"):
            display.warning(
                "An HTTPS endpoint was specified but ca_cert was not provided. "
                "The connection will be attempted without TLS. "
                "To enable TLS, provide the ca_cert option."
            )

        cnx_log = dict(client_params)
        if "password" in cnx_log:
            cnx_log["password"] = "<redacted>"
        display.verbose(f"etcd3 connection parameters: {cnx_log}")

        # connect to etcd3 server
        etcd = etcd3_client(client_params)

        ret = []
        # we can pass many keys to lookup
        for term in terms:
            if self.get_option("prefix"):
                try:
                    for val, meta in etcd.get_prefix(term):
                        if val and meta:
                            ret.append({"key": to_native(meta.key), "value": to_native(val)})
                except Exception as exp:
                    display.warning(f"Caught except during etcd3.get_prefix: {exp}")
            else:
                try:
                    val, meta = etcd.get(term)
                    if val and meta:
                        ret.append({"key": to_native(meta.key), "value": to_native(val)})
                except Exception as exp:
                    display.warning(f"Caught except during etcd3.get: {exp}")
        return ret



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_vlan_policy
short_description: Manage VLAN Policies and VLANs for Cisco Intersight
description:
  - Create, update, and delete VLAN Policies on Cisco Intersight.
  - Manage individual VLANs within VLAN policies.
  - Supports both regular VLANs and Private VLANs (Primary, Isolated, Community) configurations.
  - VLAN policies define network segmentation and can be attached to LAN Connectivity policies and Server Profiles.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/fabric/EthNetworkPolicy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the VLAN Policy.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the VLAN Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  vlans:
    description:
      - List of VLANs to be created and attached to the VLAN policy.
      - Each VLAN will be named as C(prefix_vlan_id) (e.g., prod_100).
      - Leave empty to create a policy without VLANs for manual configuration later.
    type: list
    elements: dict
    suboptions:
      prefix:
        description:
          - Prefix for the VLAN name.
          - Combined with vlan_id to create the full VLAN name (prefix_vlan_id).
        type: str
        required: true
      vlan_id:
        description:
          - Enter a valid VLAN ID or ID range between 2 and 4093.
          - You can enter a range of IDs using a hyphen (e.g., "30-40" will create VLANs 30 through 40).
          - Examples of valid VLAN IDs or ID ranges are 50, 200, "2000-2100".
          - You cannot use VLANs from 4043-4047, 4094, and 4095 because these IDs are reserved for system use.
          - You can create a maximum of 3000 VLANs at a time.
          - VLAN ID - single ID 100 or range "30-40" (ranges require quotes).
          - Must be unique within the fabric interconnect domain.
          - When using ranges, multiple VLANs will be created with names following the pattern prefix_vlanid.
          - For non-contiguous VLANs, create separate VLAN blocks rather than using comma-separated values.
        type: str
        required: true
      is_native:
        description:
          - Whether this VLAN is the native VLAN for the fabric interconnect domain.
        type: bool
        default: false
      auto_allow_on_uplinks:
        description:
          - Whether to automatically allow this VLAN on uplinks.
        type: bool
        default: true
      enable_sharing:
        description:
          - When selected, enables Private VLAN sharing options.
        type: bool
        default: false
      multicast_policy_name:
        description:
          - Name of the multicast policy to associate with this VLAN.
          - Required when enable_sharing is false.
        type: str
      sharing_type:
        description:
          - Type of VLAN sharing when enable_sharing is true.
          - Use lowercase values (primary, isolated, community).
        type: str
        choices: ['primary', 'isolated', 'community']
      primary_vlan_id:
        description:
          - The Primary VLAN ID of the VLAN, if the sharing type of the VLAN is Isolated or Community.
        type: int
      state:
        description:
          - Whether to create/update or delete the VLAN.
        type: str
        choices: ['present', 'absent']
        default: present
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a VLAN Policy with multiple VLANs
  cisco.intersight.intersight_vlan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "datacenter-vlan-policy"
    description: "VLAN policy for datacenter infrastructure"
    tags:
      - Key: "Environment"
        Value: "Production"
      - Key: "Site"
        Value: "DataCenter-A"
    vlans:
      - prefix: "prod"
        vlan_id: 100
        auto_allow_on_uplinks: true
        enable_sharing: false
        multicast_policy_name: "default-multicast-policy"
      - prefix: "dev"
        vlan_id: 200
        auto_allow_on_uplinks: false
        enable_sharing: false
        multicast_policy_name: "default-multicast-policy"
      - prefix: "mgmt"
        vlan_id: 300
        auto_allow_on_uplinks: true
        enable_sharing: false
        multicast_policy_name: "default-multicast-policy"
        is_native: true
    state: present

- name: Create a VLAN Policy with VLAN sharing (Private VLANs)
  cisco.intersight.intersight_vlan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "private-vlan-policy"
    description: "Policy with private VLAN configuration"
    vlans:
      - prefix: "primary"
        vlan_id: 79
        enable_sharing: true
        sharing_type: "primary"
        auto_allow_on_uplinks: true
      - prefix: "isolated"
        vlan_id: 90
        enable_sharing: true
        sharing_type: "isolated"
        primary_vlan_id: 79
        auto_allow_on_uplinks: true
      - prefix: "community"
        vlan_id: 91
        enable_sharing: true
        sharing_type: "community"
        primary_vlan_id: 79
        auto_allow_on_uplinks: true
    state: present

- name: Create a VLAN Policy with mixed configurations
  cisco.intersight.intersight_vlan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Engineering"
    name: "mixed-vlan-policy"
    description: "Mixed configuration with shared and non-shared VLANs"
    vlans:
      - prefix: "web"
        vlan_id: 10
        auto_allow_on_uplinks: true
        enable_sharing: false
        multicast_policy_name: "web-multicast-policy"
      - prefix: "db"
        vlan_id: 20
        auto_allow_on_uplinks: false
        enable_sharing: false
        state: absent
        multicast_policy_name: "db-multicast-policy"
      - prefix: "dmz_primary"
        vlan_id: 50
        enable_sharing: true
        sharing_type: "primary"
        auto_allow_on_uplinks: true
        state: present
      - prefix: "dmz_isolated"
        vlan_id: 51
        enable_sharing: true
        sharing_type: "isolated"
        primary_vlan_id: 50
        auto_allow_on_uplinks: true
    state: present

- name: Create a VLAN Policy with VLAN ranges
  cisco.intersight.intersight_vlan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "range-vlan-policy"
    description: "Policy with VLAN ranges"
    vlans:
      - prefix: "prod"
        vlan_id: "30-40"
        auto_allow_on_uplinks: true
        enable_sharing: false
        multicast_policy_name: "default-multicast-policy"
      - prefix: "dev"
        vlan_id: "100-110"
        auto_allow_on_uplinks: true
        enable_sharing: false
        multicast_policy_name: "default-multicast-policy"
      - prefix: "mgmt"
        vlan_id: 200
        auto_allow_on_uplinks: true
        enable_sharing: false
        multicast_policy_name: "default-multicast-policy"
    state: present

- name: Create a VLAN Policy with minimal configuration (policy only)
  cisco.intersight.intersight_vlan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "empty-vlan-policy"
    description: "Empty policy for manual VLAN configuration"
    state: present

- name: Update an existing VLAN Policy
  cisco.intersight.intersight_vlan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "datacenter-vlan-policy"
    description: "Updated description for datacenter infrastructure"
    tags:
      - Key: "Environment"
        Value: "Production"
      - Key: "Site"
        Value: "DataCenter-A"
      - Key: "Updated"
        Value: "2024-01-01"
    state: present

- name: Delete a VLAN Policy
  cisco.intersight.intersight_vlan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "datacenter-vlan-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "test_vlan_policy",
        "ObjectType": "fabric.EthNetworkPolicy",
        "Tags": [
            {
                "Key": "Site",
                "Value": "DataCenter-A"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec, compare_values


def parse_vlan_id_range(vlan_id_input):
    """
    Parse VLAN ID input which can be:
    - String with single ID: "100"
    - String with range: "100-110"
    Args:
        vlan_id_input: VLAN ID as string
    Returns:
        List of individual VLAN IDs
    Raises:
        ValueError: If the input format is invalid
    """
    vlan_ids = []
    # Parse string input
    vlan_id_str = str(vlan_id_input).strip()
    # Check for comma-separated values and reject them
    if ',' in vlan_id_str:
        raise ValueError("Comma-separated VLAN IDs are not supported. Please create separate VLAN blocks for non-contiguous VLANs")
    if '-' in vlan_id_str:
        # Range format: "100-110"
        try:
            start, end = vlan_id_str.split('-', 1)
            start_id = int(start.strip())
            end_id = int(end.strip())
            if start_id > end_id:
                raise ValueError(f"Invalid VLAN range {vlan_id_str}: start ID must be less than or equal to end ID")
            vlan_ids.extend(range(start_id, end_id + 1))
        except ValueError as e:
            if "Comma-separated" in str(e):
                raise
            raise ValueError(f"Invalid VLAN range format '{vlan_id_str}': {str(e)}")
    else:
        # Single ID
        try:
            vlan_ids.append(int(vlan_id_str))
        except ValueError:
            raise ValueError(f"Invalid VLAN ID '{vlan_id_str}': must be an integer")
    return vlan_ids


def validate_vlan_id(vlan_id):
    """
    Validate a VLAN ID is within acceptable ranges.
    Valid range: 1-4094, excluding 4043-4047, 4094, 4095
    Args:
        vlan_id: VLAN ID to validate
    Raises:
        ValueError: If VLAN ID is invalid or reserved
    Returns:
        True if valid
    """
    reserved_vlans = [4094, 4095] + list(range(4043, 4048))
    if vlan_id < 1 or vlan_id > 4094:
        raise ValueError(f"VLAN ID {vlan_id} is out of valid range (1-4094)")
    if vlan_id in reserved_vlans:
        raise ValueError(f"VLAN ID {vlan_id} is reserved for system use (4043-4047, 4094, 4095)")
    return True


def get_vlans_map_for_policy(intersight, vlan_policy_moid, top=1000):
    """
    Fetch all VLANs for a given VLAN policy and build lookup maps.

    Args:
        intersight: IntersightModule instance
        vlan_policy_moid: VLAN policy MOID
        top: Page size for pagination (will be capped to 1000)

    Returns:
        dict -> VLAN object (keyed by VlanId)
    """
    if top <= 0:
        top = 1000
    if top > 1000:
        top = 1000

    filter_str = f"EthNetworkPolicy.Moid eq '{vlan_policy_moid}'"
    resource_path = '/fabric/Vlans'

    count_options = {
        'http_method': 'get',
        'resource_path': resource_path,
        'query_params': {
            '$filter': filter_str,
            '$count': 'true'
        },
    }
    count_resp = intersight.call_api(**count_options)
    total_count = count_resp.get('Count', 0)
    if not isinstance(total_count, int) or total_count < 0:
        total_count = 0

    pages = 0
    if total_count > 0:
        # Calculate number of pages to fetch all VLANs
        pages = (total_count + top - 1) // top

    all_vlans = []
    for page_idx in range(pages):
        skip = page_idx * top
        page_options = {
            'http_method': 'get',
            'resource_path': resource_path,
            'query_params': {
                '$filter': filter_str,
                '$top': top,
                '$skip': skip,
            },
        }
        page_resp = intersight.call_api(**page_options)
        page_results = page_resp.get('Results')
        if isinstance(page_results, list) and page_results:
            all_vlans.extend(page_results)

    vlan_id_map = {}
    for vlan in all_vlans:
        vlan_id = vlan.get('VlanId')
        if isinstance(vlan_id, int):
            # VlanId is unique per policy; keep the first occurrence if duplicates appear due to paging instability.
            if vlan_id not in vlan_id_map:
                vlan_id_map[vlan_id] = vlan

    return vlan_id_map


def build_vlan_sharing_config(enable_sharing, sharing_type, primary_vlan_id, multicast_policy_moid, is_patch=False):
    """
    Build VLAN sharing configuration for POST or PATCH operations.

    Args:
        enable_sharing: Whether VLAN sharing is enabled
        sharing_type: Type of sharing (Primary, Isolated, Community)
        primary_vlan_id: Primary VLAN ID for Isolated/Community types
        multicast_policy_moid: MOID of multicast policy (if not sharing)
        is_patch: True for PATCH operations, False for POST operations

    Returns:
        Dictionary with SharingType, PrimaryVlanId, and optionally MulticastPolicy
    """
    config = {}

    if enable_sharing:
        config['SharingType'] = sharing_type
        config['PrimaryVlanId'] = primary_vlan_id if sharing_type in ['Isolated', 'Community'] else 0
        # For PATCH operations, explicitly set MulticastPolicy to None to clear it
        # when switching from non-shared to shared VLANs
        if is_patch:
            config['MulticastPolicy'] = None
    else:
        config['SharingType'] = 'None'
        config['PrimaryVlanId'] = 0
        if multicast_policy_moid:
            config['MulticastPolicy'] = multicast_policy_moid

    return config


def build_vlan_base_body(vlan_name, vlan_id, auto_allow_on_uplinks, is_native):
    """
    Build base VLAN API body with common fields.

    Args:
        vlan_name: Name of the VLAN
        vlan_id: VLAN ID
        auto_allow_on_uplinks: Whether to auto-allow on uplinks
        is_native: Whether this is the native VLAN

    Returns:
        Dictionary with base VLAN fields
    """
    return {
        'Name': vlan_name,
        'VlanId': vlan_id,
        'AutoAllowOnUplinks': auto_allow_on_uplinks,
        'IsNative': is_native
    }


def build_vlan_body_for_post(vlan_name, vlan_id, auto_allow_on_uplinks, is_native,
                             enable_sharing, sharing_type, primary_vlan_id,
                             multicast_policy_moid, vlan_policy_moid):
    """
    Build VLAN API body for POST (create) operations.

    Args:
        vlan_policy_moid: MOID of the parent VLAN policy (required for POST)

    Returns:
        Complete API body for VLAN creation
    """
    body = build_vlan_base_body(vlan_name, vlan_id, auto_allow_on_uplinks, is_native)
    body['EthNetworkPolicy'] = vlan_policy_moid
    body.update(build_vlan_sharing_config(enable_sharing, sharing_type, primary_vlan_id, multicast_policy_moid, is_patch=False))
    return body


def build_vlan_body_for_patch(vlan_name, vlan_id, auto_allow_on_uplinks, is_native,
                              enable_sharing, sharing_type, primary_vlan_id, multicast_policy_moid):
    """
    Build VLAN API body for PATCH (update) operations.

    Note: Excludes EthNetworkPolicy as it cannot be changed after creation.
    For PATCH operations, MulticastPolicy is explicitly set to None when switching
    to shared VLANs to clear any previously configured multicast policy.

    Returns:
        API body for VLAN update
    """
    body = build_vlan_base_body(vlan_name, vlan_id, auto_allow_on_uplinks, is_native)
    body.update(build_vlan_sharing_config(enable_sharing, sharing_type, primary_vlan_id, multicast_policy_moid, is_patch=True))
    return body


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        vlans=dict(type='list', elements='dict', options=dict(
            prefix=dict(type='str', required=True),
            vlan_id=dict(type='str', required=True),
            auto_allow_on_uplinks=dict(type='bool', default=True),
            enable_sharing=dict(type='bool', default=False),
            multicast_policy_name=dict(type='str'),
            sharing_type=dict(type='str', choices=['primary', 'isolated', 'community']),
            primary_vlan_id=dict(type='int'),
            is_native=dict(type='bool', default=False),
            state=dict(type='str', choices=['present', 'absent'], default='present')
        ))
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    # Initialize structured response
    final_response = {
        'vlan_policy': {},
        'vlans': []
    }

    # Initialize list to track changed states from each API call
    changed_states = []

    # Resource path used to configure policy
    resource_path = '/fabric/EthNetworkPolicies'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }
    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

    intersight.configure_policy_or_profile(resource_path=resource_path)

    # Store the VLAN policy response
    final_response['vlan_policy'] = intersight.result['api_response']

    # Save the changed state and reset for next operation
    changed_states.append(intersight.result['changed'])
    intersight.result['changed'] = False

    vlan_policy_moid = None
    if intersight.module.params['state'] == 'present' and final_response['vlan_policy']:
        vlan_policy_moid = final_response['vlan_policy'].get('Moid')

    # Process VLANs if provided
    if intersight.module.params['state'] == 'present' and intersight.module.params['vlans']:
        # Check if we have a valid VLAN policy MOID before processing VLANs
        if not vlan_policy_moid:
            if intersight.module.check_mode:
                # In check mode, policy doesn't exist yet, skip VLAN processing
                final_response['vlans'] = []
            else:
                # This shouldn't happen - policy should exist before processing VLANs
                module.fail_json(msg="VLAN policy MOID is missing, verify vlan policy was created by the module")
        else:
            # Cache for multicast policy MOIDs to avoid redundant API calls
            multicast_policy_cache = {}
            vlan_id_map = {}

            # Build all VLAN configurations and validate
            vlan_operations = {'create': [], 'update': [], 'delete': []}

            for vlan_config in intersight.module.params['vlans']:
                # Parse VLAN ID range to get list of individual VLAN IDs
                try:
                    vlan_ids = parse_vlan_id_range(vlan_config['vlan_id'])
                except ValueError as e:
                    module.fail_json(msg=f"Error parsing vlan_id '{vlan_config['vlan_id']}': {str(e)}")

                # Validate each VLAN ID
                for vlan_id in vlan_ids:
                    try:
                        validate_vlan_id(vlan_id)
                    except ValueError as e:
                        module.fail_json(msg=str(e))

                # Extract configuration
                prefix = vlan_config['prefix']
                auto_allow_on_uplinks = vlan_config.get('auto_allow_on_uplinks')
                enable_sharing = vlan_config.get('enable_sharing')
                is_native = vlan_config.get('is_native')
                vlan_state = vlan_config.get('state', 'present')

                # Process each VLAN ID in the range
                for vlan_id in vlan_ids:
                    # Generate VLAN name: prefix_vlan_id
                    vlan_name = f"{prefix}_{vlan_id}"

                    # Resolve multicast policy MOID / sharing config only when VLAN is being created/updated
                    multicast_policy_moid = None
                    sharing_type = None
                    primary_vlan_id_value = 0
                    if vlan_state == 'present':
                        if enable_sharing:
                            sharing_type_raw = vlan_config.get('sharing_type')
                            if not sharing_type_raw:
                                module.fail_json(
                                    msg=f"Configuration error for VLAN '{vlan_name}': sharing_type is required when enable_sharing is true. "
                                        f"Valid options: 'primary', 'isolated', or 'community'. "
                                        f"Example: sharing_type: 'primary'"
                                )
                            # Normalize lowercase input to proper case for API
                            sharing_type_map = {'primary': 'Primary', 'isolated': 'Isolated', 'community': 'Community'}
                            sharing_type = sharing_type_map.get(sharing_type_raw)
                            if sharing_type not in ['Primary', 'Isolated', 'Community']:
                                module.fail_json(
                                    msg=f"Configuration error for VLAN '{vlan_name}': Invalid sharing_type '{sharing_type_raw}'. "
                                        f"Valid options are: 'primary', 'isolated', or 'community'"
                                )
                            if sharing_type in ['Isolated', 'Community']:
                                if 'primary_vlan_id' not in vlan_config:
                                    module.fail_json(
                                        msg=f"Configuration error for VLAN '{vlan_name}': primary_vlan_id is required when sharing_type is '{sharing_type}'. "
                                            f"Specify the VLAN ID of the Primary VLAN. "
                                            f"Example: primary_vlan_id: 79"
                                    )
                                primary_vlan_id_value = vlan_config['primary_vlan_id']
                        else:
                            multicast_policy_name = vlan_config.get('multicast_policy_name')
                            if not multicast_policy_name:
                                module.fail_json(
                                    msg=f"Configuration error for VLAN prefix '{prefix}': multicast_policy_name is required when enable_sharing is false. "
                                        f"Specify the name of an existing multicast policy. "
                                        f"Example: multicast_policy_name: 'default'"
                                )
                            if multicast_policy_name in multicast_policy_cache:
                                multicast_policy_moid = multicast_policy_cache[multicast_policy_name]
                            else:
                                multicast_policy_moid = intersight.get_moid_by_name_and_org(
                                    resource_path='/fabric/MulticastPolicies',
                                    resource_name=multicast_policy_name,
                                    organization_name=intersight.module.params['organization']
                                )
                                if not multicast_policy_moid:
                                    module.fail_json(
                                        msg=f"Multicast policy '{multicast_policy_name}' not found in organization '{intersight.module.params['organization']}'"
                                    )
                                multicast_policy_cache[multicast_policy_name] = multicast_policy_moid

                    # Build VLAN-ID map only once (after we know the policy MOID and before we start checking VLANs)
                    if not vlan_id_map:
                        vlan_id_map = get_vlans_map_for_policy(
                            intersight=intersight,
                            vlan_policy_moid=vlan_policy_moid,
                            top=1000
                        )

                    # Check if VLAN already exists using cached map (VlanId is unique per policy)
                    existing_vlan = vlan_id_map.get(vlan_id, {})

                    existing_moid = existing_vlan.get('Moid')
                    custom_filter = f"VlanId eq {vlan_id} and EthNetworkPolicy.Moid eq '{vlan_policy_moid}'"

                    # Determine operation type
                    if vlan_state == 'present':
                        if existing_moid:
                            # VLAN exists, check if update is needed
                            # Build body for PATCH (without EthNetworkPolicy, proper object references)
                            vlan_patch_body = build_vlan_body_for_patch(
                                vlan_name=vlan_name,
                                vlan_id=vlan_id,
                                auto_allow_on_uplinks=auto_allow_on_uplinks,
                                is_native=is_native,
                                enable_sharing=enable_sharing,
                                sharing_type=sharing_type,
                                primary_vlan_id=primary_vlan_id_value,
                                multicast_policy_moid=multicast_policy_moid
                            )
                            if not compare_values(vlan_patch_body, existing_vlan):
                                vlan_operations['update'].append({
                                    'body': vlan_patch_body,
                                    'moid': existing_moid,
                                    'name': vlan_name,
                                    'filter': custom_filter
                                })
                            else:
                                # VLAN exists and matches, add to response but no change needed
                                final_response['vlans'].append(existing_vlan)
                        else:
                            # VLAN doesn't exist, needs to be created
                            # Build body for POST (includes EthNetworkPolicy)
                            vlan_post_body = build_vlan_body_for_post(
                                vlan_name=vlan_name,
                                vlan_id=vlan_id,
                                auto_allow_on_uplinks=auto_allow_on_uplinks,
                                is_native=is_native,
                                enable_sharing=enable_sharing,
                                sharing_type=sharing_type,
                                primary_vlan_id=primary_vlan_id_value,
                                multicast_policy_moid=multicast_policy_moid,
                                vlan_policy_moid=vlan_policy_moid
                            )
                            vlan_operations['create'].append({
                                'body': vlan_post_body,
                                'name': vlan_name,
                                'filter': custom_filter
                            })
                    else:  # state == 'absent'
                        if existing_moid:
                            vlan_operations['delete'].append({
                                'moid': existing_moid,
                                'name': vlan_name,
                                'body': {
                                    'VlanId': vlan_id
                                }
                            })

            # Execute bulk operations
            bulk_results = intersight.execute_bulk_operations(
                resource_path='/fabric/Vlans',
                operations_dict=vlan_operations,
                changed_states=changed_states
            )
            final_response['vlans'].extend(bulk_results)

    # Set the final structured response
    intersight.result['api_response'] = final_response

    # Set final changed state based on whether any operation resulted in a change
    intersight.result['changed'] = any(changed_states)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2016-2017, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2017, Markus Teufelberger <mteufelberger+ansible@mgit.at>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Standard files documentation fragment
    DOCUMENTATION = r"""
description:
  - This module allows one to (re)generate OpenSSL certificates.
  - It uses the cryptography python library to interact with OpenSSL.
attributes:
  diff_mode:
    support: full
  idempotent:
    support: partial
    details:
      - If relative timestamps are used and O(ignore_timestamps=false), the module is not idempotent.
      - The option O(force=true) generally disables idempotency.
requirements:
  - cryptography >= 3.3 (if using V(selfsigned) or V(ownca) provider)
options:
  force:
    description:
      - Generate the certificate, even if it already exists.
    type: bool
    default: false

  csr_path:
    description:
      - Path to the Certificate Signing Request (CSR) used to generate this certificate.
      - This is mutually exclusive with O(csr_content).
    type: path
  csr_content:
    description:
      - Content of the Certificate Signing Request (CSR) used to generate this certificate.
      - This is mutually exclusive with O(csr_path).
    type: str

  privatekey_path:
    description:
      - Path to the private key to use when signing the certificate.
      - This is mutually exclusive with O(privatekey_content).
    type: path
  privatekey_content:
    description:
      - Content of the private key to use when signing the certificate.
      - This is mutually exclusive with O(privatekey_path).
    type: str

  privatekey_passphrase:
    description:
      - The passphrase for the O(privatekey_path) resp. O(privatekey_content).
      - This is required if the private key is password protected.
    type: str

  ignore_timestamps:
    description:
      - Whether the "not before" and "not after" timestamps should be ignored for idempotency checks.
      - It is better to keep the default value V(true) when using relative timestamps (like V(+0s) for now).
    type: bool
    default: true
    version_added: 2.0.0

  select_crypto_backend:
    description:
      - Determines which crypto backend to use.
      - The default choice is V(auto), which tries to use C(cryptography) if available.
      - If set to V(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
      - Note that with community.crypto 3.0.0, all values behave the same.
        This option will be deprecated in a later version.
        We recommend to not set it explicitly.
    type: str
    default: auto
    choices: [auto, cryptography]

notes:
  - All ASN.1 TIME values should be specified following the YYYYMMDDHHMMSSZ pattern.
  - Date specified should be UTC. Minutes and seconds are mandatory.
  - For security reason, when you use V(ownca) provider, you should NOT run M(community.crypto.x509_certificate) on a target
    machine, but on a dedicated CA machine. It is recommended not to store the CA private key on the target machine. Once
    signed, the certificate can be moved to the target machine.
seealso:
  - module: community.crypto.openssl_csr
  - module: community.crypto.openssl_csr_pipe
  - module: community.crypto.openssl_dhparam
  - module: community.crypto.openssl_pkcs12
  - module: community.crypto.openssl_privatekey
  - module: community.crypto.openssl_privatekey_pipe
  - module: community.crypto.openssl_publickey
"""

    BACKEND_ACME_DOCUMENTATION = r"""
description:
  - This module allows one to (re)generate OpenSSL certificates.
requirements:
  - acme-tiny >= 4.0.0 (if using the V(acme) provider)
options:
  acme_accountkey_path:
    description:
      - The path to the accountkey for the V(acme) provider.
      - This is only used by the V(acme) provider.
    type: path

  acme_challenge_path:
    description:
      - The path to the ACME challenge directory that is served on U(http://<HOST>:80/.well-known/acme-challenge/)
      - This is only used by the V(acme) provider.
    type: path

  acme_chain:
    description:
      - Include the intermediate certificate to the generated certificate
      - This is only used by the V(acme) provider.
      - Note that this is only available for older versions of C(acme-tiny).
        New versions include the chain automatically, and setting O(acme_chain) to V(true) results in an error.
    type: bool
    default: false

  acme_directory:
    description:
      - "The ACME directory to use. You can use any directory that supports the ACME protocol, such as Let's Encrypt."
      - "Let's Encrypt recommends using their staging server while developing jobs. U(https://letsencrypt.org/docs/staging-environment/)."
    type: str
    default: https://acme-v02.api.letsencrypt.org/directory
"""

    BACKEND_OWNCA_DOCUMENTATION = r"""
description:
  - The V(ownca) provider is intended for generating an OpenSSL certificate signed with your own
    CA (Certificate Authority) certificate (self-signed certificate).
options:
  ownca_path:
    description:
      - Remote absolute path of the CA (Certificate Authority) certificate.
      - This is only used by the V(ownca) provider.
      - This is mutually exclusive with O(ownca_content).
    type: path
  ownca_content:
    description:
      - Content of the CA (Certificate Authority) certificate.
      - This is only used by the V(ownca) provider.
      - This is mutually exclusive with O(ownca_path).
    type: str

  ownca_privatekey_path:
    description:
      - Path to the CA (Certificate Authority) private key to use when signing the certificate.
      - This is only used by the V(ownca) provider.
      - This is mutually exclusive with O(ownca_privatekey_content).
    type: path
  ownca_privatekey_content:
    description:
      - Content of the CA (Certificate Authority) private key to use when signing the certificate.
      - This is only used by the V(ownca) provider.
      - This is mutually exclusive with O(ownca_privatekey_path).
    type: str

  ownca_privatekey_passphrase:
    description:
      - The passphrase for the O(ownca_privatekey_path) resp. O(ownca_privatekey_content).
      - This is only used by the V(ownca) provider.
    type: str

  ownca_digest:
    description:
      - The digest algorithm to be used for the V(ownca) certificate.
      - This is only used by the V(ownca) provider.
    type: str
    default: sha256

  ownca_version:
    description:
      - The version of the V(ownca) certificate.
      - Nowadays it should almost always be V(3).
      - This is only used by the V(ownca) provider.
    type: int
    default: 3
    choices:
      - 3

  ownca_not_before:
    description:
      - The point in time the certificate is valid from.
      - Time can be specified either as relative time or as absolute timestamp.
      - Time will always be interpreted as UTC.
      - Valid format is C([+-]timespec | ASN.1 TIME) where timespec can be an integer
        + C([w | d | h | m | s]) (for example V(+32w1d2h)).
      - If this value is not specified, the certificate will start being valid from now.
      - Note that this value is B(not used to determine whether an existing certificate should be regenerated).
        This can be changed by setting the O(ignore_timestamps) option to V(false). Please note that you should
        avoid relative timestamps when setting O(ignore_timestamps=false).
      - This is only used by the V(ownca) provider.
    type: str
    default: +0s

  ownca_not_after:
    description:
      - The point in time at which the certificate stops being valid.
      - Time can be specified either as relative time or as absolute timestamp.
      - Time will always be interpreted as UTC.
      - Valid format is C([+-]timespec | ASN.1 TIME) where timespec can be an integer
        + C([w | d | h | m | s]) (for example V(+32w1d2h)).
      - If this value is not specified, the certificate will stop being valid 10 years from now.
      - Note that this value is B(not used to determine whether an existing certificate should be regenerated).
        This can be changed by setting the O(ignore_timestamps) option to V(false). Please note that you should
        avoid relative timestamps when setting O(ignore_timestamps=false).
      - This is only used by the V(ownca) provider.
      - On macOS 10.15 and onwards, TLS server certificates must have a validity period of 825 days or fewer.
        Please see U(https://support.apple.com/en-us/HT210176) for more details.
    type: str
    default: +3650d

  ownca_create_subject_key_identifier:
    description:
      - Whether to create the Subject Key Identifier (SKI) from the public key.
      - A value of V(create_if_not_provided) (default) only creates a SKI when the CSR does not
        provide one.
      - A value of V(always_create) always creates a SKI. If the CSR provides one, that one is
        ignored.
      - A value of V(never_create) never creates a SKI. If the CSR provides one, that one is used.
      - This is only used by the V(ownca) provider.
    type: str
    choices: [create_if_not_provided, always_create, never_create]
    default: create_if_not_provided

  ownca_create_authority_key_identifier:
    description:
      - Create a Authority Key Identifier from the CA's certificate. If the CSR provided
        a authority key identifier, it is ignored.
      - The Authority Key Identifier is generated from the CA certificate's Subject Key Identifier,
        if available. If it is not available, the CA certificate's public key will be used.
      - This is only used by the V(ownca) provider.
    type: bool
    default: true
"""

    BACKEND_SELFSIGNED_DOCUMENTATION = r"""
notes:
  - For the V(selfsigned) provider, O(csr_path) and O(csr_content) are optional. If not provided, a
    certificate without any information (Subject, Subject Alternative Names, Key Usage, etc.) is created.

options:
  # NOTE: descriptions in options are overwritten, not appended. For that reason, the texts provided
  #       here for csr_path and csr_content are not visible to the user. That's why this information is
  #       added to the notes (see above).

  # csr_path:
  #   description:
  #     - This is optional for the V(selfsigned) provider. If not provided, a certificate
  #       without any information (Subject, Subject Alternative Names, Key Usage, etc.) is
  #       created.

  # csr_content:
  #   description:
  #     - This is optional for the V(selfsigned) provider. If not provided, a certificate
  #       without any information (Subject, Subject Alternative Names, Key Usage, etc.) is
  #       created.

  selfsigned_version:
    description:
      - Version of the V(selfsigned) certificate.
      - Nowadays it should almost always be V(3).
      - This is only used by the V(selfsigned) provider.
    type: int
    default: 3
    choices:
      - 3

  selfsigned_digest:
    description:
      - Digest algorithm to be used when self-signing the certificate.
      - This is only used by the V(selfsigned) provider.
    type: str
    default: sha256

  selfsigned_not_before:
    description:
      - The point in time the certificate is valid from.
      - Time can be specified either as relative time or as absolute timestamp.
      - Time will always be interpreted as UTC.
      - Valid format is C([+-]timespec | ASN.1 TIME) where timespec can be an integer
        + C([w | d | h | m | s]) (for example V(+32w1d2h)).
      - If this value is not specified, the certificate will start being valid from now.
      - Note that this value is B(not used to determine whether an existing certificate should be regenerated).
        This can be changed by setting the O(ignore_timestamps) option to V(false). Please note that you should
        avoid relative timestamps when setting O(ignore_timestamps=false).
      - This is only used by the V(selfsigned) provider.
    type: str
    default: +0s
    aliases:
      - selfsigned_notBefore

  selfsigned_not_after:
    description:
      - The point in time at which the certificate stops being valid.
      - Time can be specified either as relative time or as absolute timestamp.
      - Time will always be interpreted as UTC.
      - Valid format is C([+-]timespec | ASN.1 TIME) where timespec can be an integer
        + C([w | d | h | m | s]) (for example V(+32w1d2h)).
      - If this value is not specified, the certificate will stop being valid 10 years from now.
      - Note that this value is B(not used to determine whether an existing certificate should be regenerated).
        This can be changed by setting the O(ignore_timestamps) option to V(false). Please note that you should
        avoid relative timestamps when setting O(ignore_timestamps=false).
      - This is only used by the V(selfsigned) provider.
      - On macOS 10.15 and onwards, TLS server certificates must have a validity period of 825 days or fewer.
        Please see U(https://support.apple.com/en-us/HT210176) for more details.
    type: str
    default: +3650d
    aliases:
      - selfsigned_notAfter

  selfsigned_create_subject_key_identifier:
    description:
      - Whether to create the Subject Key Identifier (SKI) from the public key.
      - A value of V(create_if_not_provided) (default) only creates a SKI when the CSR does not
        provide one.
      - A value of V(always_create) always creates a SKI. If the CSR provides one, that one is
        ignored.
      - A value of V(never_create) never creates a SKI. If the CSR provides one, that one is used.
      - This is only used by the V(selfsigned) provider.
    type: str
    choices: [create_if_not_provided, always_create, never_create]
    default: create_if_not_provided
"""



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios_evpn_evi config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.evpn_evi import (
    Evpn_eviTemplate,
)


class Evpn_evi(ResourceModule):
    """
    The ios_evpn_evi config class
    """

    def __init__(self, module):
        super(Evpn_evi, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="evpn_evi",
            tmplt=Evpn_eviTemplate(),
        )
        self.parsers = [
            "default_gateway.advertise.enable",
            "default_gateway.advertise.disable",
            "encapsulation",
            "ip.local_learning.enable",
            "ip.local_learning.disable",
            "replication_type",
            "route_distinguisher",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = {entry["evi"]: entry for entry in self.want}
        haved = {entry["evi"]: entry for entry in self.have}

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = {k: v for k, v in haved.items() if k in wantd or not wantd}
            wantd = {}

        # remove superfluous config for overridden and deleted
        if self.state in ["overridden", "deleted"]:
            for k, have in haved.items():
                if k not in wantd:
                    self.addcmd(have, "evi", negate=True)

        for k, want in wantd.items():
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Evpn_evi network resource.
        """
        begin = len(self.commands)
        self.compare(parsers=self.parsers, want=want, have=have)
        if len(self.commands) != begin:
            self.commands.insert(begin, self._tmplt.render(want or have, "evi", False))



#!/usr/bin/python

# Copyright Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: cisco_webex
short_description: Send a message to a Cisco Webex Teams Room or Individual
description:
  - Send a message to a Cisco Webex Teams Room or Individual with options to control the formatting.
author: Drew Rusell (@drew-russell)
notes:
  - The O(recipient_type) must be valid for the supplied O(recipient_id).
  - Full API documentation can be found at U(https://developer.webex.com/docs/api/basics).
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:

  recipient_type:
    description:
      - The request parameter you would like to send the message to.
      - Messages can be sent to either a room or individual (by ID or E-Mail).
    required: true
    choices: ['roomId', 'toPersonEmail', 'toPersonId']
    type: str

  recipient_id:
    description:
      - The unique identifier associated with the supplied O(recipient_type).
    required: true
    type: str

  msg_type:
    description:
      - Specifies how you would like the message formatted.
    default: text
    choices: ['text', 'markdown']
    type: str
    aliases: ['message_type']

  personal_token:
    description:
      - Your personal access token required to validate the Webex Teams API.
    required: true
    aliases: ['token']
    type: str

  msg:
    description:
      - The message you would like to send.
    required: true
    type: str
"""

EXAMPLES = r"""
# Note: The following examples assume a variable file has been imported
# that contains the appropriate information.

- name: Cisco Webex Teams - Markdown Message to a Room
  community.general.cisco_webex:
    recipient_type: roomId
    recipient_id: "{{ room_id }}"
    msg_type: markdown
    personal_token: "{{ token }}"
    msg: "**Cisco Webex Teams Ansible Module - Room Message in Markdown**"

- name: Cisco Webex Teams - Text Message to a Room
  community.general.cisco_webex:
    recipient_type: roomId
    recipient_id: "{{ room_id }}"
    msg_type: text
    personal_token: "{{ token }}"
    msg: "Cisco Webex Teams Ansible Module - Room Message in Text"

- name: Cisco Webex Teams - Text Message by an Individuals ID
  community.general.cisco_webex:
    recipient_type: toPersonId
    recipient_id: "{{ person_id}}"
    msg_type: text
    personal_token: "{{ token }}"
    msg: "Cisco Webex Teams Ansible Module - Text Message to Individual by ID"

- name: Cisco Webex Teams - Text Message by an Individuals E-Mail Address
  community.general.cisco_webex:
    recipient_type: toPersonEmail
    recipient_id: "{{ person_email }}"
    msg_type: text
    personal_token: "{{ token }}"
    msg: "Cisco Webex Teams Ansible Module - Text Message to Individual by E-Mail"
"""

RETURN = r"""
status_code:
  description:
    - The Response Code returned by the Webex Teams API.
    - Full Response Code explanations can be found at U(https://developer.webex.com/docs/api/basics).
  returned: always
  type: int
  sample: 200

message:
  description:
    - The Response Message returned by the Webex Teams API.
    - Full Response Code explanations can be found at U(https://developer.webex.com/docs/api/basics).
  returned: always
  type: str
  sample: OK (585 bytes)
"""
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


def webex_msg(module):
    """When check mode is specified, establish a read only connection, that does not return any user specific
    data, to validate connectivity. In regular mode, send a message to a Cisco Webex Teams Room or Individual"""

    # Ansible Specific Variables
    results = {}
    ansible = module.params

    headers = {"Authorization": f"Bearer {ansible['personal_token']}", "content-type": "application/json"}

    if module.check_mode:
        url = "https://webexapis.com/v1/people/me"
        payload = None

    else:
        url = "https://webexapis.com/v1/messages"

        payload = {ansible["recipient_type"]: ansible["recipient_id"], ansible["msg_type"]: ansible["msg"]}

        payload = module.jsonify(payload)

    response, info = fetch_url(module, url, data=payload, headers=headers)

    status_code = info["status"]
    msg = info["msg"]

    # Module will fail if the response is not 200
    if status_code != 200:
        results["failed"] = True
        results["status_code"] = status_code
        results["message"] = msg
    else:
        results["failed"] = False
        results["status_code"] = status_code

        if module.check_mode:
            results["message"] = "Authentication Successful."
        else:
            results["message"] = msg

    return results


def main():
    """Ansible main."""
    module = AnsibleModule(
        argument_spec=dict(
            recipient_type=dict(required=True, choices=["roomId", "toPersonEmail", "toPersonId"]),
            recipient_id=dict(required=True, no_log=True),
            msg_type=dict(default="text", aliases=["message_type"], choices=["text", "markdown"]),
            personal_token=dict(required=True, no_log=True, aliases=["token"]),
            msg=dict(required=True),
        ),
        supports_check_mode=True,
    )

    results = webex_msg(module)

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_settings
short_description: Resource module for networks _switch _settings
description:
  - Manage operation update of the resource networks _switch _settings.
  - Update switch network settings.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  macBlocklist:
    description: MAC blocklist.
    suboptions:
      enabled:
        description: Enable MAC blocklist.
        type: bool
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  powerExceptions:
    description: Exceptions on a per switch basis to "useCombinedPower".
    elements: dict
    suboptions:
      powerType:
        description: Per switch exception (combined, redundant, useNetworkSetting).
        type: str
      serial:
        description: Serial number of the switch.
        type: str
    type: list
  uplinkClientSampling:
    description: Uplink client sampling.
    suboptions:
      enabled:
        description: Enable uplink client sampling.
        type: bool
    type: dict
  uplinkSelection:
    description: Settings related to uplink selection on IOS-XE switches.
    suboptions:
      candidates:
        description: '''all'' lets devices try any potential interface. ''designated''
          restricts to specified candidates (configured via the Routing & DHCP page).'
        type: str
      failback:
        description: If a preferred uplink ever goes offline, we will periodically
          reattempt it.
        suboptions:
          enabled:
            description: Enable preferred uplink failback.
            type: bool
        type: dict
    type: dict
  useCombinedPower:
    description: The use Combined Power as the default behavior of secondary power
      supplies on supported devices.
    type: bool
  vlan:
    description: Management VLAN.
    type: int
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch updateNetworkSwitchSettings
    description: Complete reference of the updateNetworkSwitchSettings API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-settings
notes:
  - SDK Method used are
    switch.Switch.update_network_switch_settings,
  - Paths used are
    put /networks/{networkId}/switch/settings,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_switch_settings:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    macBlocklist:
      enabled: true
    networkId: string
    powerExceptions:
      - powerType: string
        serial: string
    uplinkClientSampling:
      enabled: true
    uplinkSelection:
      candidates: string
      failback:
        enabled: true
    useCombinedPower: true
    vlan: 0
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "vlan": 0,
      "useCombinedPower": true,
      "powerExceptions": [
        {
          "serial": "string",
          "powerType": "string"
        }
      ],
      "uplinkClientSampling": {
        "enabled": true
      },
      "macBlocklist": {
        "enabled": true
      },
      "uplinkSelection": {
        "failback": {
          "enabled": true
        },
        "candidates": "string"
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2018, Simon Weald <ansible@simonweald.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: memset_memstore_info
author: "Simon Weald (@glitchcrab)"
short_description: Retrieve Memstore product usage information
notes:
  - An API key generated using the Memset customer control panel is needed with the following minimum scope - C(memstore.usage).
description:
  - Retrieve Memstore product usage information.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
attributes:
  check_mode:
    version_added: 3.3.0
    # This was backported to 2.5.4 and 1.3.11 as well, since this was a bugfix
options:
  api_key:
    required: true
    type: str
    description:
      - The API key obtained from the Memset control panel.
  name:
    required: true
    type: str
    description:
      - The Memstore product name (that is, V(mstestyaa1)).
"""

EXAMPLES = r"""
- name: Get usage for mstestyaa1
  community.general.memset_memstore_info:
    name: mstestyaa1
    api_key: 5eb86c9896ab03919abcf03857163741
  delegate_to: localhost
"""

RETURN = r"""
memset_api:
  description: Info from the Memset API.
  returned: always
  type: complex
  contains:
    cdn_bandwidth:
      description: Dictionary of CDN bandwidth facts.
      returned: always
      type: complex
      contains:
        bytes_out:
          description: Outbound CDN bandwidth for the last 24 hours in bytes.
          returned: always
          type: int
          sample: 1000
        requests:
          description: Number of requests in the last 24 hours.
          returned: always
          type: int
          sample: 10
        bytes_in:
          description: Inbound CDN bandwidth for the last 24 hours in bytes.
          returned: always
          type: int
          sample: 1000
    containers:
      description: Number of containers.
      returned: always
      type: int
      sample: 10
    bytes:
      description: Space used in bytes.
      returned: always
      type: int
      sample: 3860997965
    objs:
      description: Number of objects.
      returned: always
      type: int
      sample: 1000
    bandwidth:
      description: Dictionary of CDN bandwidth facts.
      returned: always
      type: complex
      contains:
        bytes_out:
          description: Outbound bandwidth for the last 24 hours in bytes.
          returned: always
          type: int
          sample: 1000
        requests:
          description: Number of requests in the last 24 hours.
          returned: always
          type: int
          sample: 10
        bytes_in:
          description: Inbound bandwidth for the last 24 hours in bytes.
          returned: always
          type: int
          sample: 1000
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._memset import memset_api_call


def get_facts(args=None):
    """
    Performs a simple API call and returns a JSON blob.
    """
    retvals, payload = dict(), dict()
    has_changed, has_failed = False, False

    payload["name"] = args["name"]

    api_method = "memstore.usage"
    has_failed, msg, response = memset_api_call(api_key=args["api_key"], api_method=api_method, payload=payload)

    if has_failed:
        # this is the first time the API is called; incorrect credentials will
        # manifest themselves at this point so we need to ensure the user is
        # informed of the reason.
        retvals["failed"] = has_failed
        retvals["msg"] = msg
        if response.status_code is not None:
            retvals["stderr"] = f"API returned an error: {response.status_code}"
        else:
            retvals["stderr"] = f"{response.stderr}"
        return retvals

    # we don't want to return the same thing twice
    memset_api = response.json()

    retvals["changed"] = has_changed
    retvals["failed"] = has_failed
    retvals["msg"] = None
    retvals["memset_api"] = memset_api

    return retvals


def main():
    global module
    module = AnsibleModule(
        argument_spec=dict(api_key=dict(required=True, type="str", no_log=True), name=dict(required=True, type="str")),
        supports_check_mode=True,
    )

    # populate the dict with the user-provided vars.
    args = dict(module.params)

    retvals = get_facts(args)

    if retvals["failed"]:
        module.fail_json(**retvals)
    else:
        module.exit_json(**retvals)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_action_batches_info
short_description: Information module for organizations _action _batches
description:
  - Get all organizations _action _batches.
  - Get organizations _action _batches by id.
  - Return an action batch.
  - Return the list of action batches in the organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  status:
    description:
      - Status query parameter. Filter batches by status. Valid types are pending,
        completed, and failed.
    type: str
  actionBatchId:
    description:
      - ActionBatchId path parameter. Action batch ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationActionBatch
    description: Complete reference of the getOrganizationActionBatch API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-action-batch
  - name: Cisco Meraki documentation for organizations getOrganizationActionBatches
    description: Complete reference of the getOrganizationActionBatches API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-action-batches
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_action_batch,
    organizations.Organizations.get_organization_action_batches,
  - Paths used are
    get /organizations/{organizationId}/actionBatches,
    get /organizations/{organizationId}/actionBatches/{actionBatchId},
"""

EXAMPLES = r"""
- name: Get all organizations _action _batches
  cisco.meraki.organizations_action_batches_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    status: string
    organizationId: string
  register: result
- name: Get organizations _action _batches by id
  cisco.meraki.organizations_action_batches_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    organizationId: string
    actionBatchId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "organizationId": "string",
      "confirmed": true,
      "synchronous": true,
      "status": {
        "completed": true,
        "failed": true,
        "errors": [
          "string"
        ],
        "createdResources": [
          {
            "id": "string",
            "uri": "string"
          }
        ]
      },
      "actions": [
        {
          "resource": "string",
          "operation": "string",
          "body": {}
        }
      ],
      "callback": {
        "id": "string",
        "url": "string",
        "status": "string"
      }
    }
"""



# Copyright (c) 2025, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Felix Fontein (@felixfontein)
name: tasks_only
type: stdout
version_added: 11.1.0
short_description: Only show tasks
description:
  - Removes play start and stats marker from P(ansible.builtin.default#callback)'s output.
  - Can be used to generate output for documentation examples.
    For this, the O(number_of_columns) option should be set to an explicit value.
extends_documentation_fragment:
  - ansible.builtin.default_callback
  - ansible.builtin.result_format_callback
options:
  number_of_columns:
    description:
      - Sets the number of columns for Ansible's display.
    type: int
    env:
      - name: ANSIBLE_COLLECTIONS_TASKS_ONLY_NUMBER_OF_COLUMNS
  result_format:
    # Part of the ansible.builtin.result_format_callback doc fragment
    version_added: 11.2.0
  pretty_results:
    # Part of the ansible.builtin.result_format_callback doc fragment
    version_added: 11.2.0
"""

EXAMPLES = r"""
---
# Enable callback in ansible.cfg:
ansible_config: |-
  [defaults]
  stdout_callback = community.general.tasks_only

---
# Enable callback with environment variables:
environment_variable: |-
  ANSIBLE_STDOUT_CALLBACK=community.general.tasks_only
"""

from ansible.plugins.callback.default import CallbackModule as Default


class CallbackModule(Default):
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "stdout"
    CALLBACK_NAME = "community.general.tasks_only"

    def v2_playbook_on_play_start(self, play):
        pass

    def v2_playbook_on_stats(self, stats):
        pass

    def set_options(self, *args, **kwargs):
        result = super().set_options(*args, **kwargs)
        self.number_of_columns = self.get_option("number_of_columns")
        if self.number_of_columns is not None:
            self._display.columns = self.number_of_columns
        return result



# Copyright (c) 2021, Cliff Hults <cliff.hlts@gmail.com>
# Copyright (c) 2021 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
name: icinga2
short_description: Icinga2 inventory source
version_added: 3.7.0
author:
  - Cliff Hults (@BongoEADGC6) <cliff.hults@gmail.com>
description:
  - Get inventory hosts from the Icinga2 API.
  - Uses a configuration file as an inventory source, it must end in C(.icinga2.yml) or C(.icinga2.yaml).
extends_documentation_fragment:
  - ansible.builtin.constructed
options:
  strict:
    version_added: 4.4.0
  compose:
    version_added: 4.4.0
  groups:
    version_added: 4.4.0
  keyed_groups:
    version_added: 4.4.0
  plugin:
    description: Name of the plugin.
    required: true
    type: string
    choices: ['community.general.icinga2']
  url:
    description: Root URL of Icinga2 API.
    type: string
    required: true
  user:
    description: Username to query the API.
    type: string
    required: true
  password:
    description: Password to query the API.
    type: string
    required: true
  host_filter:
    description:
      - An Icinga2 API valid host filter. Leave blank for no filtering.
    type: string
    required: false
  validate_certs:
    description: Enables or disables SSL certificate verification.
    type: boolean
    default: true
  inventory_attr:
    description:
      - Allows the override of the inventory name based on different attributes.
      - This allows for changing the way limits are used.
      - The current default, V(address), is sometimes not unique or present. We recommend to use V(name) instead.
    type: string
    default: address
    choices: ['name', 'display_name', 'address']
    version_added: 4.2.0
  group_by_hostgroups:
    description:
      - Uses Icinga2 hostgroups as groups.
    type: boolean
    default: true
    version_added: 8.4.0
"""

EXAMPLES = r"""
# my.icinga2.yml
plugin: community.general.icinga2
url: http://localhost:5665
user: ansible
password: secure
host_filter: \"linux-servers\" in host.groups
validate_certs: false  # only do this when connecting to localhost!
inventory_attr: name
groups:
  # simple name matching
  webservers: inventory_hostname.startswith('web')

  # using icinga2 template
  databaseservers: "'db-template' in (icinga2_attributes.templates|list)"

compose:
  # set all icinga2 attributes to a host variable 'icinga2_attrs'
  icinga2_attrs: icinga2_attributes

  # set 'ansible_user' and 'ansible_port' from icinga2 host vars
  ansible_user: icinga2_attributes.vars.ansible_user
  ansible_port: icinga2_attributes.vars.ansible_port | default(22)
"""

import json
from http import HTTPStatus
from urllib.error import HTTPError

from ansible.errors import AnsibleParserError
from ansible.module_utils.urls import open_url
from ansible.plugins.inventory import BaseInventoryPlugin, Constructable

from ansible_collections.community.general.plugins.plugin_utils._unsafe import make_unsafe


class InventoryModule(BaseInventoryPlugin, Constructable):
    """Host inventory parser for ansible using Icinga2 as source."""

    NAME = "community.general.icinga2"

    def __init__(self):
        super().__init__()

        # from config
        self.icinga2_url = None
        self.icinga2_user = None
        self.icinga2_password = None
        self.ssl_verify = None
        self.host_filter = None
        self.inventory_attr = None
        self.group_by_hostgroups = None

        self.cache_key = None
        self.use_cache = None

    def verify_file(self, path):
        valid = False
        if super().verify_file(path):
            if path.endswith(("icinga2.yaml", "icinga2.yml")):
                valid = True
            else:
                self.display.vvv('Skipping due to inventory source not ending in "icinga2.yaml" nor "icinga2.yml"')
        return valid

    def _api_connect(self):
        self.headers = {
            "User-Agent": "ansible-icinga2-inv",
            "Accept": "application/json",
        }
        api_status_url = f"{self.icinga2_url}/status"
        request_args = {
            "headers": self.headers,
            "url_username": self.icinga2_user,
            "url_password": self.icinga2_password,
            "validate_certs": self.ssl_verify,
        }
        open_url(api_status_url, **request_args)

    def _post_request(self, request_url, data=None):
        self.display.vvv(f"Requested URL: {request_url}")
        request_args = {
            "headers": self.headers,
            "url_username": self.icinga2_user,
            "url_password": self.icinga2_password,
            "validate_certs": self.ssl_verify,
        }
        if data is not None:
            request_args["data"] = json.dumps(data)
        self.display.vvv(f"Request Args: {request_args}")
        try:
            response = open_url(request_url, **request_args)
        except HTTPError as e:
            try:
                error_body = json.loads(e.read().decode())
                self.display.vvv(f"Error returned: {error_body}")
            except Exception:
                error_body = {"status": None}
            if e.code == HTTPStatus.NOT_FOUND and error_body.get("status") == "No objects found.":
                raise AnsibleParserError(
                    "Host filter returned no data. Please confirm your host_filter value is valid"
                ) from e
            raise AnsibleParserError(f"Unexpected data returned: {e} -- {error_body}") from e

        response_body = response.read()
        json_data = json.loads(response_body.decode("utf-8"))
        self.display.vvv(f"Returned Data: {json.dumps(json_data, indent=4, sort_keys=True)}")
        if HTTPStatus.OK <= response.status < HTTPStatus.MULTIPLE_CHOICES:  # 2xx codes
            return json_data
        if response.status == HTTPStatus.NOT_FOUND and json_data["status"] == "No objects found.":
            raise AnsibleParserError(f"API returned no data -- Response: {response.status} - {json_data['status']}")
        if response.status == HTTPStatus.UNAUTHORIZED:
            raise AnsibleParserError(
                f"API was unable to complete query -- Response: {response.status} - {json_data['status']}"
            )
        if response.status == HTTPStatus.INTERNAL_SERVER_ERROR:
            raise AnsibleParserError(f"API Response - {json_data['status']} - {json_data['errors']}")
        raise AnsibleParserError(f"Unexpected data returned - {json_data['status']} - {json_data['errors']}")

    def _query_hosts(self, hosts=None, attrs=None, joins=None, host_filter=None):
        query_hosts_url = f"{self.icinga2_url}/objects/hosts"
        self.headers["X-HTTP-Method-Override"] = "GET"
        data_dict = dict()
        if hosts:
            data_dict["hosts"] = hosts
        if attrs is not None:
            data_dict["attrs"] = attrs
        if joins is not None:
            data_dict["joins"] = joins
        if host_filter is not None:
            data_dict["filter"] = host_filter.replace('\\"', '"')
            self.display.vvv(host_filter)
        host_dict = self._post_request(query_hosts_url, data_dict)
        return host_dict["results"]

    def get_inventory_from_icinga(self):
        """Query for all hosts"""
        self.display.vvv("Querying Icinga2 for inventory")
        query_args = {
            "attrs": [
                "address",
                "address6",
                "name",
                "display_name",
                "state_type",
                "state",
                "templates",
                "groups",
                "vars",
                "zone",
            ],
        }
        if self.host_filter is not None:
            query_args["host_filter"] = self.host_filter
        # Icinga2 API Call
        results_json = self._query_hosts(**query_args)
        # Manipulate returned API data to Ansible inventory spec
        ansible_inv = self._convert_inv(results_json)
        return ansible_inv

    def _apply_constructable(self, name, variables):
        strict = self.get_option("strict")
        self._add_host_to_composed_groups(self.get_option("groups"), variables, name, strict=strict)
        self._add_host_to_keyed_groups(self.get_option("keyed_groups"), variables, name, strict=strict)
        self._set_composite_vars(self.get_option("compose"), variables, name, strict=strict)

    def _populate(self):
        groups = self._to_json(self.get_inventory_from_icinga())
        return groups

    def _to_json(self, in_dict):
        """Convert dictionary to JSON"""
        return json.dumps(in_dict, sort_keys=True, indent=2)

    def _convert_inv(self, json_data):
        """Convert Icinga2 API data to JSON format for Ansible"""
        groups_dict = {"_meta": {"hostvars": {}}}
        for entry in json_data:
            host_attrs = make_unsafe(entry["attrs"])
            if self.inventory_attr == "name":
                host_name = make_unsafe(entry.get("name"))
            if self.inventory_attr == "address":
                # When looking for address for inventory, if missing fallback to object name
                if host_attrs.get("address", "") != "":
                    host_name = make_unsafe(host_attrs.get("address"))
                else:
                    host_name = make_unsafe(entry.get("name"))
            if self.inventory_attr == "display_name":
                host_name = host_attrs.get("display_name")
            if host_attrs["state"] == 0:
                host_attrs["state"] = "on"
            else:
                host_attrs["state"] = "off"
            self.inventory.add_host(host_name)
            if self.group_by_hostgroups:
                host_groups = host_attrs.get("groups")
                for group in host_groups:
                    if group not in self.inventory.groups.keys():
                        self.inventory.add_group(group)
                    self.inventory.add_child(group, host_name)
            # If the address attribute is populated, override ansible_host with the value
            if host_attrs.get("address") != "":
                self.inventory.set_variable(host_name, "ansible_host", host_attrs.get("address"))
            self.inventory.set_variable(host_name, "hostname", make_unsafe(entry.get("name")))
            self.inventory.set_variable(host_name, "display_name", host_attrs.get("display_name"))
            self.inventory.set_variable(host_name, "state", host_attrs["state"])
            self.inventory.set_variable(host_name, "state_type", host_attrs["state_type"])
            # Adds all attributes to a variable 'icinga2_attributes'
            construct_vars = dict(self.inventory.get_host(host_name).get_vars())
            construct_vars["icinga2_attributes"] = host_attrs
            self._apply_constructable(host_name, construct_vars)
        return groups_dict

    def parse(self, inventory, loader, path, cache=True):
        super().parse(inventory, loader, path)

        # read config from file, this sets 'options'
        self._read_config_data(path)

        # Store the options from the YAML file
        self.icinga2_url = self.get_option("url")
        self.icinga2_user = self.get_option("user")
        self.icinga2_password = self.get_option("password")
        self.ssl_verify = self.get_option("validate_certs")
        self.host_filter = self.get_option("host_filter")
        self.inventory_attr = self.get_option("inventory_attr")
        self.group_by_hostgroups = self.get_option("group_by_hostgroups")

        if self.templar.is_template(self.icinga2_url):
            self.icinga2_url = self.templar.template(variable=self.icinga2_url)
        if self.templar.is_template(self.icinga2_user):
            self.icinga2_user = self.templar.template(variable=self.icinga2_user)
        if self.templar.is_template(self.icinga2_password):
            self.icinga2_password = self.templar.template(variable=self.icinga2_password)

        self.icinga2_url = f"{self.icinga2_url.rstrip('/')}/v1"

        # Not currently enabled
        # self.cache_key = self.get_cache_key(path)
        # self.use_cache = cache and self.get_option('cache')

        # Test connection to API
        self._api_connect()

        # Call our internal helper to populate the dynamic inventory
        self._populate()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage Check Point Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_package
short_description: Manages package objects on Check Point over Web Services API
description:
  - Manages package objects on Check Point devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "1.0.0"
author: "Or Soffer (@chkp-orso)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  access:
    description:
      - True - enables, False - disables access & NAT policies, empty - nothing is changed.
    type: bool
  desktop_security:
    description:
      - True - enables, False - disables Desktop security policy, empty - nothing is changed.
    type: bool
  installation_targets:
    description:
      - Which Gateways identified by the name or UID to install the policy on.
    type: list
    elements: str
  qos:
    description:
      - True - enables, False - disables QoS policy, empty - nothing is changed.
    type: bool
  qos_policy_type:
    description:
      - QoS policy type.
    type: str
    choices: ['recommended', 'express']
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  threat_prevention:
    description:
      - True - enables, False - disables Threat policy, empty - nothing is changed.
    type: bool
  vpn_traditional_mode:
    description:
      - True - enables, False - disables VPN traditional mode, empty - nothing is changed.
    type: bool
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
  access_layers:
    description:
      - Access policy layers.
    type: dict
    suboptions:
      add:
        description:
          - Collection of Access layer objects to be added identified by the name or UID.
        type: list
        elements: dict
        suboptions:
          name:
            description:
              - Layer name or UID.
            type: str
          position:
            description:
              - Layer position.
            type: int
      remove:
        description:
          - Collection of Access layer objects to be removed identified by the name or UID.
        type: list
        elements: str
      value:
        description:
          - Collection of Access layer objects to be set identified by the name or UID. Replaces existing Access layers.
        type: list
        elements: str
  threat_layers:
    description:
      - Threat policy layers.
    type: dict
    suboptions:
      add:
        description:
          - Collection of Threat layer objects to be added identified by the name or UID.
        type: list
        elements: dict
        suboptions:
          name:
            description:
              - Layer name or UID.
            type: str
          position:
            description:
              - Layer position.
            type: int
      remove:
        description:
          - Collection of Threat layer objects to be removed identified by the name or UID.
        type: list
        elements: str
      value:
        description:
          - Collection of Threat layer objects to be set identified by the name or UID. Replaces existing Threat layers.
        type: list
        elements: str
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-package
  cp_mgmt_package:
    access: true
    color: green
    comments: My Comments
    name: New_Standard_Package_1
    state: present
    threat_prevention: false

- name: set-package
  cp_mgmt_package:
    access_layers:
      add:
        - name: New Access Layer 1
          position: 1
    name: Standard
    state: present
    threat_layers:
      add:
        - name: New Layer 1
          position: 2

- name: delete-package
  cp_mgmt_package:
    name: New Standard Package 1
    state: absent
"""

RETURN = """
cp_mgmt_package:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        access=dict(type="bool"),
        desktop_security=dict(type="bool"),
        installation_targets=dict(type="list", elements="str"),
        qos=dict(type="bool"),
        qos_policy_type=dict(type="str", choices=["recommended", "express"]),
        tags=dict(type="list", elements="str"),
        threat_prevention=dict(type="bool"),
        vpn_traditional_mode=dict(type="bool"),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
        access_layers=dict(
            type="dict",
            options=dict(
                add=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        name=dict(type="str"), position=dict(type="int")
                    ),
                ),
                remove=dict(type="list", elements="str"),
                value=dict(type="list", elements="str"),
            ),
        ),
        threat_layers=dict(
            type="dict",
            options=dict(
                add=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        name=dict(type="str"), position=dict(type="int")
                    ),
                ),
                remove=dict(type="list", elements="str"),
                value=dict(type="list", elements="str"),
            ),
        ),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "package"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_licensing_coterm_licenses_move
short_description: Resource module for organizations _licensing _coterm _licenses
  _move
description:
  - Manage operation create of the resource organizations _licensing _coterm _licenses
    _move.
  - Moves a license to a different organization coterm only .
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  destination:
    description: Destination data for the license move.
    suboptions:
      mode:
        description: The claim mode of the moved license.
        type: str
      organizationId:
        description: The organization to move the license to.
        type: str
    type: dict
  licenses:
    description: The list of licenses to move.
    elements: dict
    suboptions:
      counts:
        description: The counts to move from the license by model type.
        elements: dict
        suboptions:
          count:
            description: The number of counts to move.
            type: int
          model:
            description: The license model type to move counts of.
            type: str
        type: list
      key:
        description: The license key to move counts from.
        type: str
    type: list
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for licensing moveOrganizationLicensingCotermLicenses
    description: Complete reference of the moveOrganizationLicensingCotermLicenses
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!move-organization-licensing-coterm-licenses
notes:
  - SDK Method used are
    licensing.Licensing.move_organization_licensing_coterm_licenses,
  - Paths used are
    post /organizations/{organizationId}/licensing/coterm/licenses/move,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_licensing_coterm_licenses_move:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    destination:
      mode: addDevices
      organizationId: '123'
    licenses:
      - counts:
          - count: 5
            model: MR Enterprise
        key: Z2AA-BBBB-CCCC
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "remainderLicenses": [
        {
          "key": "string",
          "organizationId": "string",
          "duration": 0,
          "mode": "string",
          "startedAt": "string",
          "claimedAt": "string",
          "invalidated": true,
          "invalidatedAt": "string",
          "expired": true,
          "editions": [
            {
              "edition": "string",
              "productType": "string"
            }
          ],
          "counts": [
            {
              "model": "string",
              "count": 0
            }
          ]
        }
      ],
      "movedLicenses": [
        {
          "key": "string",
          "organizationId": "string",
          "duration": 0,
          "mode": "string",
          "startedAt": "string",
          "claimedAt": "string",
          "invalidated": true,
          "invalidatedAt": "string",
          "expired": true,
          "editions": [
            {
              "edition": "string",
              "productType": "string"
            }
          ],
          "counts": [
            {
              "model": "string",
              "count": 0
            }
          ]
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for cp_mgmt_add_access_layers
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
module: cp_mgmt_access_layers
short_description: Manages ACCESS LAYERS resource module
description:
  - This resource module allows for addition, deletion, or modification of CP Access Layers.
  - This resource module also takes care of gathering Access layer config facts
  - Available from R80 management version.
version_added: "5.0.0"
author: Ansible Security Automation Team (@justjais) <https://github.com/ansible-security>
options:
  config:
    description: A dictionary of ACCESS LAYERS options
    type: dict
    suboptions:
      name:
        description: Object name. Must be unique in the domain.
        type: str
      add_default_rule:
        description: Indicates whether to include a cleanup rule in the new layer.
        type: bool
      applications_and_url_filtering:
        description: Whether to enable Applications & URL Filtering blade on the layer.
        type: bool
      content_awareness:
        description: Whether to enable Content Awareness blade on the layer.
        type: bool
      detect_using_x_forward_for:
        description: Whether to use X-Forward-For HTTP header, which is added by the  proxy
          server to keep track of the original source IP.
        type: bool
      firewall:
        description: Whether to enable Firewall blade on the layer.
        type: bool
      implicit_cleanup_action:
        description: The default "catch-all" action for traffic that does not match
          any explicit or implied rules in the layer.
        type: str
        choices:
        - drop
        - accept
      mobile_access:
        description: Whether to enable Mobile Access blade on the layer.
        type: bool
      shared:
        description: Whether this layer is shared.
        type: bool
      tags:
        description: Collection of tag identifiers.
        type: list
        elements: str
      color:
        description: Color of the object. Should be one of existing colors.
        type: str
        choices:
        - aquamarine
        - black
        - blue
        - crete blue
        - burlywood
        - cyan
        - dark green
        - khaki
        - orchid
        - dark orange
        - dark sea green
        - pink
        - turquoise
        - dark blue
        - firebrick
        - brown
        - forest green
        - gold
        - dark gold
        - gray
        - dark gray
        - light green
        - lemon chiffon
        - coral
        - sea green
        - sky blue
        - magenta
        - purple
        - slate blue
        - violet red
        - navy blue
        - olive
        - orange
        - red
        - sienna
        - yellow
      comments:
        description: Comments string.
        type: str
      details_level:
        description: The level of detail for some of the fields in the response can
          vary from showing only the UID value of the object to a fully detailed representation
          of the object.
        type: str
        choices:
        - uid
        - standard
        - full
      ignore_warnings:
        description: Apply changes ignoring warnings.
        type: bool
      ignore_errors:
        description: Apply changes ignoring errors. You won't be able to publish such
          a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
        type: bool
      limit:
        description:
          - The maximal number of returned results.
          - NOTE, this parameter is a valid parameter only for the GATHERED state, for config states
            like, MERGED, REPLACED, and DELETED state it won't be applicable.
        type: int
      offset:
        description:
          - Number of the results to initially skip.
          - NOTE, this parameter is a valid parameter only for the GATHERED state, for config states
            like, MERGED, REPLACED, and DELETED state it won't be applicable.
        type: int
      order:
        description:
          - Sorts results by the given field. By default the results are sorted in the ascending order by name.
            This parameter is relevant only for getting few objects.
          - NOTE, this parameter is a valid parameter only for the GATHERED state, for config states
            like, MERGED, REPLACED, and DELETED state it won't be applicable.
        type: list
        elements: dict
        suboptions:
          ASC:
            description:
              - Sorts results by the given field in ascending order.
            type: str
          DESC:
            description:
              - Sorts results by the given field in descending order.
            type: str
      round_trip:
        description:
          - If set to True, the round trip will filter out the module parameters from the response param,
            which will enable the user to fire the config request using the structured gathered data.
          - NOTE, this parameter makes relevance only with the GATHERED state, as for config states like,
            MERGED, REPLACED, and DELETED state it won't make any config updates,
            as it's not a module config parameter.
        type: bool
      auto_publish_session:
        description:
          - Publish the current session if changes have been performed
            after task completes.
        type: bool
      version:
        description:
          - Version of checkpoint. If not given one, the latest version taken.
        type: str
  state:
    description:
    - The state the configuration should be left in
    - The state I(gathered) will get the module API configuration from the device
      and transform it into structured data in the format as per the module argspec
      and the value is returned in the I(gathered) key within the result.
    type: str
    choices:
    - merged
    - replaced
    - gathered
    - deleted
"""

EXAMPLES = """

# Using MERGED state
# -------------------

- name: Merge Access-layer config
  cp_mgmt_access_layers:
    state: merged
    config:
      name: New Layer 1
      add_default_rule: true
      applications_and_url_filtering: true
      content_awareness: true
      detect_using_x_forward_for: false
      firewall: true
      implicit_cleanup_action: drop
      mobile_access: true
      shared: false
      tags:
        - test_layer
      color: aquamarine
      comments: test description
      details_level: full
      ignore_warnings: false
      ignore_errors: false
      round_trip: true

# RUN output:
# -----------

# mgmt_access_layers:
#   after:
#     applications_and_url_filtering: true
#     color: aquamarine
#     comments: test description
#     content_awareness: true
#     detect_using_x_forward_for: false
#     domain: SMC User
#     firewall: true
#     icon: ApplicationFirewall/rulebase
#     implicit_cleanup_action: drop
#     mobile_access: true
#     name: New Layer 1
#     shared: false
#     tags:
#     - test_layer
#     uid: eb74d7fe-81a6-4e6c-aedb-d2d6599f965e
#   before: {}

# Using REPLACED state
# --------------------

- name: Replace Access-layer config
  cp_mgmt_access_layers:
    state: replaced
    config:
      name: New Layer 1
      add_default_rule: true
      applications_and_url_filtering: true
      content_awareness: false
      detect_using_x_forward_for: false
      firewall: true
      implicit_cleanup_action: drop
      mobile_access: true
      shared: true
      tags:
        - test_layer_replaced
      color: cyan
      comments: test REPLACE description
      details_level: full
      ignore_warnings: false
      ignore_errors: false
      round_trip: true

# RUN output:
# -----------

# mgmt_access_layers:
#   after:
#     applications_and_url_filtering: true
#     color: cyan
#     comments: test REPLACE description
#     content_awareness: false
#     detect_using_x_forward_for: false
#     domain: SMC User
#     firewall: true
#     icon: ApplicationFirewall/sharedrulebase
#     implicit_cleanup_action: drop
#     mobile_access: true
#     name: New Layer 1
#     shared: true
#     tags:
#     - test_layer_replaced
#     uid: a4e2bbc1-ec94-4b85-9b00-07ad1279ac12
#   before:
#     applications_and_url_filtering: true
#     color: aquamarine
#     comments: test description
#     content_awareness: true
#     detect_using_x_forward_for: false
#     firewall: true
#     icon: ApplicationFirewall/rulebase
#     implicit_cleanup_action: drop
#     mobile_access: true
#     name: New Layer 1
#     shared: false
#     tags:
#     - test_layer

# Using GATHERED state
# --------------------

# 1. With Round Trip set to True

- name: Gather Access-layers config by Name
  cp_mgmt_access_layers:
    state: gathered
    config:
      name: New Layer 1
      round_trip: true

# RUN output:
# -----------

# gathered:
#   applications_and_url_filtering: true
#   color: aquamarine
#   comments: test description
#   content_awareness: true
#   detect_using_x_forward_for: false
#   domain: SMC User
#   firewall: true
#   icon: ApplicationFirewall/rulebase
#   implicit_cleanup_action: drop
#   mobile_access: true
#   name: New Layer 1
#   shared: false
#   tags:
#   - test_layer
#   uid: eb74d7fe-81a6-4e6c-aedb-d2d6599f965e

# 2. With Round Trip set to False which is the default behaviour

- name: Gather Access-layers config by Name
  cp_mgmt_access_layers:
    state: gathered
    config:
      name: New Layer 1

# RUN output:
# -----------

# gathered:
#   applications_and_url_filtering: true
#   color: turquoise
#   comments: test description
#   content_awareness: true
#   detect_using_x_forward_for: false
#   domain:
#     domain-type: domain
#     name: SMC User
#     uid: 41e821a0-3720-11e3-aa6e-0800200c9fde
#   firewall: true
#   icon: ApplicationFirewall/rulebase
#   implicit_cleanup_action: drop
#   meta-info:
#     creation-time:
#       iso-8601: 2022-11-21T07:34+0000
#       posix: 1669016073937
#     creator: admin
#     last-modifier: admin
#     last-modify-time:
#       iso-8601: 2022-11-21T07:34+0000
#       posix: 1669016074765
#     lock: unlocked
#     validation-state: ok
#   mobile_access: true
#   name: New Layer 1
#   read-only: false
#   shared: false
#   tags:
#   - domain:
#       domain-type: domain
#       name: SMC User
#       uid: 41e821a0-3720-11e3-aa6e-0800200c9fde
#     name: test_layer
#     type: tag
#     uid: 22cc8b0d-984f-47de-b1f6-276b3377eb0c
#   type: access-layer
#   uid: a54e47d3-22fc-4aff-90d9-f644aa4a1522

# 3. Gather ALL threat-layer config with DESC order filter

- name: To Gather ALL access-layer and order by Name
  cp_mgmt_access_layers:
    config:
      order:
        - DESC: name
    state: gathered

# RUN output:
# -----------

# gathered:
#   - domain:
#       domain-type: domain
#       name: SMC User
#       uid: 41e821a0-3720-11e3-aa6e-0800200c9fde
#     name: New Layer 1
#     type: access-layer
#     uid: a54e47d3-22fc-4aff-90d9-f644aa4a1522
#   - domain:
#       domain-type: domain
#       name: SMC User
#       uid: 41e821a0-3720-11e3-aa6e-0800200c9fde
#     name: Network
#     type: access-layer
#     uid: 63b7fe60-76d2-4287-bca5-21af87337b0a

# Using DELETED state
# -------------------

- name: Delete Access-layer config by Name
  cp_mgmt_access_layers:
    state: deleted
    config:
      name: New Layer 1

# RUN output:
# -----------

# mgmt_access_layers:
#   after: {}
#   before:
#     applications_and_url_filtering: true
#     color: cyan
#     comments: test REPLACE description
#     content_awareness: false
#     detect_using_x_forward_for: false
#     domain: SMC User
#     firewall: true
#     icon: ApplicationFirewall/sharedrulebase
#     implicit_cleanup_action: drop
#     mobile_access: true
#     name: New Layer 1
#     shared: true
#     tags:
#     - test_layer_replaced
#     uid: a4e2bbc1-ec94-4b85-9b00-07ad1279ac12
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when state is I(merged), I(replaced), I(deleted)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when state is I(gathered)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
"""
